/************************************************************************/
/* ople.h								*/
/*                                                                      */
/* Header file for the central scrutinizer.				*/
/************************************************************************/
/*                                                                      */
/*     Center for High Angular Resolution Astronomy                     */
/* Georgia State University, Atlanta GA 30303-3083, U.S.A.              */
/*                                                                      */
/*                                                                      */
/* Telephone: 1-626-796-5405                                            */
/* Fax      : 1-626-796-6717                                            */
/* email    : theo@chara.gsu.edu                                        */
/* WWW      : http://www.chara.gsu.edu/~theo/theo.html                  */
/*                                                                      */
/* (C) This source code and its associated executable                   */
/* program(s) are copyright.                                            */
/*                                                                      */
/************************************************************************/
/*                                                                      */
/* Author : Theo ten Brummelaar                                         */
/* Date   : SEPTEMBER 1998                                              */
/************************************************************************/

#ifndef __OPLE__
#define __OPLE__

//#define USE_ARC_CCD
#define USE_VIS_DITHER
/* #include <charaui.h> */
/* #include <chara.h> */
/* #include <clock.h> */
/* #include <rwfits.h> */
/* #include <baytech.h> */
/* #include <nsimpleX.h> */
/* #ifdef USE_ARC_CCD */
/* #include "../arcccd/arc_ccd.h" */
/* #endif */
/* #ifdef USE_VIS_DITHER */
/* #include "../vis_dither/vis_dither.h" */
/* #endif */
/* #include "../../cliserv/include/chara_messages.h" */
/* #include "./include/dlCommands.h" */
/* #include "./include/typeTag.h" */
/* #include "./include/rtModNum.h" */
/* #include "./include/dlErrors.h" */
/* #include "./include/instError.h" */
#include "./dlCommands.h"

/* #warning KLUDGE CHANGE ON DATA_DIRECTORY */
/* #define DATA_DIRECTORY "/ople/data/" */

/* #ifdef SAVE */
/* FILE * test_output; */
/* #endif */

/* Macros */

#define OPLE_TO_CHARA_TICK ((float)TICKS_PER_SECOND/\
				(float)TIME_TICKS_PER_SECOND)
#define ople_to_chara_time(t) (CHARA_TIME)((float)t*OPLE_TO_CHARA_TICK+0.5)
#define chara_to_ople_time(t) (OPLE_TIME)((float)t/OPLE_TO_CHARA_TICK+0.5)

#define OPLE_MACHINE	"vme1.chara-array.org"
#define OPLE_SOCKET	5001

#define byte_swap(a) reverse_byte_order(&(a),sizeof(a))

#define NOT_MOVING	0
#define MOVING_FORWARD	1
#define MOVING_BACKWARD	2
#define FORCED_FORWARD	3
#define FORCED_BACKWARD	4

/* What is the maximum number of data chanels we would allow? */

#define MAX_DATA_CHANNELS	2

/* So lengths make sense */

#define LASER_SIGN -1                           

/* Info about the metrology */

#define NUM_LASER_PASSES 4.0 /* How many times laser goes through optics */
#define NUM_LASER_TICKS  256.0 /* How many ticks per wavelength */

/*
 * The following is a bit of a cludge as it hardwires the
 * laser wavelength. Never the less it helps in the display of
 * position information. It also does not take into account the
 * refractive index of the air. However this way a human can understand
 * the displayed numbers in the status screen.
 */

#define LASER_TO_METERS  (1.319e-6/(NUM_LASER_PASSES*NUM_LASER_TICKS))
#define LASER_TO_FEET	 (3.2808398950*LASER_TO_METERS)

/* How many files are we allowed to make ? */

#define MAX_FILE_NUMBER 999

/* 
 * Globals
 */

/* Is this the primary server? */

extern bool primary_server;

/*
 * Declaration of Delay Line Gizmo ID array
 * Automatically generated by make_gizmoid_array.c
 * run on the Sparc and compile in C++
 * This program uses code in the real-time system
 * and should be run again if the system changes and
 * the OPLE system (and FT?) recompiled also.
 */

extern int delay_line_gizmo_id[NUM_DELAY_LINES];

extern int laser_sign[NUM_DELAY_LINES];

extern int instrument_gizmo_id;

/* The socket we will use for comms */

extern int ople_socket;

/* Various globals for strings and things */

/* extern char    *ople_target_types[TT_NUM_DL_TARGETS+1]; */

/* extern char    *ople_on_off_types[3]; */

/* extern char    *ople_modulation_types[MT_WOBBLE+2]; */

/* extern char    *delay_command_status[DLCS_SET_DITHER_COMPLETE+2]; */

/* extern char    *delay_servo_mode[DM_USER2+2]; */

/* extern FILE    *ople_data; /\* Used for saving status info as it runs *\/ */

/* extern FILE    *log_file; /\* For logging OPLE commands *\/ */

/* extern bool    save_ople_data; */

/* extern char	ople_current_direction[NUM_DELAY_LINES]; */

/* extern char	ople_skip_cable[NUM_DELAY_LINES]; */

/*
 * Global array for status informtation
 */

/* extern tDlStatus dl_status[NUM_DELAY_LINES]; */

/* Global for saving ople data at 5kHz rate in a file */

extern int	ople_data_fp[NUM_DELAY_LINES];
extern int 	data_count;


/* Stuff for running mean of motor velocity for cable puller routine */

#define    MOTOR_PERIOD  4
#define    MOTOR_THRESH  0.0
#define	   MOTOR_SPEED_THRESH	20
extern int motor_cycle[NUM_DELAY_LINES];
extern float motor_running_mean[NUM_DELAY_LINES][MOTOR_PERIOD];
extern float motor_mean_speed[NUM_DELAY_LINES];

/* Global for talking to the Baytech RPC unit */

extern struct sserial rpc_port;

/* Global bools */

extern bool sending_open_loop_targets;
extern bool data_socket_needs_flushing;
extern bool show_error_messages;
extern bool cart_active[NUM_DELAY_LINES];
extern bool show_all_carts;

/* Status info for delay line */

/* extern char *delay_command_status[DLCS_SET_DITHER_COMPLETE+2]; */
extern  struct error_messages_struct {                
	int number;               
	char *string;
        } error_messages[];

/* Globals for preserving the last commanded targets */

extern double open_loop_position[NUM_DELAY_LINES];
extern double open_loop_rate[NUM_DELAY_LINES];
extern OPLE_TIME open_loop_time[NUM_DELAY_LINES];
extern double manual_position[NUM_DELAY_LINES];
extern double manual_rate[NUM_DELAY_LINES];
extern OPLE_TIME manual_time[NUM_DELAY_LINES];
extern bool open_loop_on[NUM_DELAY_LINES];
extern bool manual_on[NUM_DELAY_LINES];

/* Global to see which type of status window we want */

#define OPLE_STATUS     0
#define ARC_CCD_STATUS  1

extern int current_status;

/* PROTOTYPES */

/* ople.c */

//int main(int argc, char **argv);

/* ople_background.c */

//void setup_ople_background_jobs(void);
//void unsetup_ople_background_jobs(void);
//#ifdef USE_ARC_CCD
//int swap_backgrounds(int argc, char **argv);
//#endif
/* int ople_top_job(void); */
/* int linux_time_status(void); */
/* int rt_time_status(void); */
/* int ople_time_status(void); */
/* int process_incomming_data(void); */
/* int delay_line_status(void); */
/* int delay_line_dir_status(void); */
/* int flush_data_socket(void); */
/* void clear_ople_ping(void); */
/* bool ople_pinged(void); */

/* ople_comms.c */

bool open_ople_socket(void);
void close_ople_socket(void);
bool send_ople_command(ISIT_CMD command, int iGizmoId);
bool ople_select_target(int dl_number, int target, bool set_active);
bool ople_dither_on_off(int dl_number, bool on);
bool ople_data_suspend(int dl_number, bool suspend);
bool ople_open_loop_target(int dl_number, double position, double rate,
	OPLE_TIME time);
bool ople_manual_target(int dl_number, double position, double rate,
	OPLE_TIME time);
bool ople_set_time(int next_second);
bool ople_off(int dl_number);
bool wait_for_stop(int dl_number);
bool ople_all_off(void);
bool ople_front(int dl_number);
bool ople_back(int dl_number);
bool ople_track(int dl_number);
bool ople_user1(int dl_number);
bool ople_user2(int dl_number);
bool ople_home_check(int dl_number);
bool ople_home_set(int dl_number);
bool ople_dither_calibrate(int dl_number);
bool ople_dither_check(int dl_number);
bool ople_ping(int dl_number);
bool ople_clr_laser_ctr(int dl_number);
bool ople_modulate(int dl_number, int mod_type, double frequency, 
	double amplitude, double mean_value, OPLE_TIME sync_time);
bool ople_set_dither(int dl_number, double amplitude, double period);
bool ople_set_servo_params(int dl_number, tGuiGsuDlServoParameters params);
void add_to_log(int dl_number,char *command);
int incriment_manual_target(int cart, double inc);
 
/* ople_control.c */

int call_open_ople_socket(int argc, char **argv);
int call_close_ople_socket(int argc, char **argv);
int reopen_ople_socket(int argc, char **argv);
int get_dl_number(int argc, char **argv);
int call_ople_select_target(int argc, char **argv);
int call_ople_dither_on_off(int argc, char **argv);
int start_ople_data(int argc, char **argv);
int stop_ople_data(int argc, char **argv);
int call_ople_open_loop_target(int argc, char **argv);
int call_ople_manual_target(int argc, char **argv);
int metric_ople_open_loop_target(int argc, char **argv);
int metric_ople_manual_target(int argc, char **argv);
int feet_ople_open_loop_target(int argc, char **argv);
int feet_ople_manual_target(int argc, char **argv);
int zero_all_targets(int argc, char **argv);
int zero_all_targets_all_carts(int argc, char **argv);
int call_ople_off(int argc, char **argv);
int call_ople_all_off(int argc, char **argv);
int call_ople_front(int argc, char **argv);
int call_ople_back(int argc, char **argv);
int call_ople_track(int argc, char **argv);
int all_track(int argc, char **argv);
int call_ople_user1(int argc, char **argv);
int call_ople_user2(int argc, char **argv);
int call_ople_home_check(int argc, char **argv);
int call_ople_home_set(int argc, char **argv);
int call_ople_dither_calibrate(int argc, char **argv);
int call_ople_dither_check(int argc, char **argv);
int call_ople_ping(int argc, char **argv);
int call_ople_clr_laser_ctr(int argc, char **argv);
int call_ople_set_time(int argc, char **argv);
int sync_ople_time_to_chara_time(int argc, char **argv);
int call_ople_modulate(int argc, char **argv);
int call_ople_set_dither(int argc, char **argv);
int set_error_messages(int argc, char **argv);
int toggle_logfile(int argc, char **argv);
int test_ople_manual_target(int argc, char **argv);
int call_clear_ople_ping(int argc, char **argv);
int call_ople_pinged(int argc, char **argv);
int star_name(int argc, char **argv);
int cart_error(int argc, char **argv);
int current_manual(int argc, char **argv);
int call_incriment_manual_target(int argc, char **argv);
int control_cart_active(int argc, char **argv);
int set_slew_velocity(int argc, char **argv);
int set_show_all_carts(int argc, char **argv);

/* ople_align.c */

int slew_test(int argc, char **argv);
int align_cart(int argc, char **argv);
int monitor_positions(int argc, char **argv);
int monitor_status(int argc, char **argv);
int slew(int argc, char **argv);
int dog_pony(int argc, char **argv);
int s1_s2(int argc, char **argv);
int start_log(int argc, char *argv[]);
int stop_log(int argc, char *argv[]);
int get_data(int argc, char **argv);
int time_step(int argc, char **argv);
int halt_cart(int argc, char **argv);
int scan(int argc, char **argv);
int search_chanel(int argc, char **argv);
int search_fluor(int argc, char **argv);

/* ople_astromod.c */

int start_astromod(int argc, char **argv);
bool get_next_data(void);
int astromod_background(void);
int stop_astromod(int argc, char **argv);
int set_reference_cart(int argc, char **argv);
int set_send_offset(int argc, char **argv);
int offset_target(int argc, char **argv);
int auto_reference(int argc, char **argv);
int chara_number(int argc, char **argv);

/* ople_messages.c */

void setup_standard_ople_messages(void);
int open_ople_socket_message(struct smessage *message);
int close_ople_socket_message(struct smessage *message);
int select_target_message(struct smessage *message);
int open_loop_target_message(struct smessage *message);
int manual_target_message(struct smessage *message);
int cart_off_message(struct smessage *message);
int all_carts_off_message(struct smessage *message);
int cart_front_message(struct smessage *message);
int cart_back_message(struct smessage *message);
int cart_track_message(struct smessage *message);
int cart_home_message(struct smessage *message);
int cart_clear_laser_message(struct smessage *message);
int start_astromod_message(struct smessage *message);
int stop_astromod_message(struct smessage *message);
int set_reference_cart_message(struct smessage *message);
int current_select_targets_message(struct smessage *message);
int current_manual_targets_message(struct smessage *message);
int current_open_loop_targets_message(struct smessage *message);
int start_scan_message(struct smessage *message);
int metrology_data_on_message(struct smessage *message);
int metrology_data_off_message(struct smessage *message);
int sync_ople_message(struct smessage *message);
int log_file_message(struct smessage *message);
int run_offset_message(struct smessage *message);
int tracking_error_message(struct smessage *message);
int astromod_running_message(struct smessage *message);
int auto_reference_message(struct smessage *message);
int limits_status_message(struct smessage *message);
int carts_moving_message(struct smessage *message);
int manual_offsets_message(struct smessage *message);

/* ople_servo.c */

int set_vc2_gain(int argc, char **argv);
int show_servo_params(int argc, char **argv);

#endif
